# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
from trytond.model import ModelView, ModelSQL, fields, Unique
from trytond.pyson import Eval, Not, Bool, Equal
from trytond.pool import Pool
from trytond import backend
from trytond.transaction import Transaction
from sql import *
from sql.aggregate import *


__all__ = ['PatientPregnancy', 'PrenatalEvaluation', 'PuerperiumMonitor',
    'Perinatal', 'PerinatalMonitor', 'GnuHealthPatient',
    'PatientMenstrualHistory', 'PatientMammographyHistory',
    'PatientPAPHistory', 'PatientColposcopyHistory']


class PatientPregnancy(ModelSQL, ModelView):
    'Patient Pregnancy'
    __name__ = 'gnuhealth.patient.pregnancy'

    name = fields.Many2One('gnuhealth.patient', 'Patient ID')
    gravida = fields.Integer('Pregnancy #', required=True)
    warning = fields.Boolean('Warn', help='Check this box if this is pregancy'
        ' is or was NOT normal')
    warning_icon = fields.Function(fields.Char('Pregnancy warning icon'), 'get_warn_icon')
    reverse = fields.Boolean ('Reverse', help="Use this method *only* when the \
        pregnancy information is referred by the patient, as a history taking \
        procedure. Please keep in mind that the reverse pregnancy data is \
        subjective",
        states={
            'invisible': Bool(Eval('current_pregnancy')),
            }
        )
    reverse_weeks = fields.Integer ("Pr. Weeks",help="Number of weeks at \
        the end of pregnancy. Used only with the reverse input method.",
        states={
        'invisible': Not(Bool(Eval('reverse'))),
        'required': Bool(Eval('reverse')),
            }
        )
    lmp = fields.Date('LMP', help="Last Menstrual Period", required=True)

    pdd = fields.Function(fields.Date('Pregnancy Due Date'),
        'get_pregnancy_data')
    prenatal_evaluations = fields.One2Many(
        'gnuhealth.patient.prenatal.evaluation', 'name',
        'Prenatal Evaluations')
    perinatal = fields.One2Many('gnuhealth.perinatal', 'name',
        'Perinatal Info')
    puerperium_monitor = fields.One2Many('gnuhealth.puerperium.monitor',
        'name', 'Puerperium monitor')
    current_pregnancy = fields.Boolean('Current Pregnancy', help='This field'
        ' marks the current pregnancy')
    fetuses = fields.Integer('Fetuses', required=True)
    monozygotic = fields.Boolean('Monozygotic')
    pregnancy_end_result = fields.Selection([
        (None, ''),
        ('live_birth', 'Live birth'),
        ('abortion', 'Abortion'),
        ('stillbirth', 'Stillbirth'),
        ('status_unknown', 'Status unknown'),
        ], 'Result', sort=False,
            states={
            'invisible': Bool(Eval('current_pregnancy')),
            'required': Not(Bool(Eval('current_pregnancy'))),
            })
    pregnancy_end_date = fields.DateTime('End of Pregnancy',
        states={
            'invisible': Bool(Eval('current_pregnancy')),
            'required': Not(Bool(Eval('current_pregnancy'))),
            })
    bba = fields.Boolean('BBA', help="Born Before Arrival",
        states={
            'invisible': Bool(Eval('current_pregnancy')),
            })
    home_birth = fields.Boolean('Home Birth', help="Home Birth",
        states={
            'invisible': Bool(Eval('current_pregnancy')),
            })

    pregnancy_end_age = fields.Function(fields.Char('Weeks', help='Weeks at'
        ' the end of pregnancy'), 'get_pregnancy_data')
    iugr = fields.Selection([
        (None, ''),
        ('symmetric', 'Symmetric'),
        ('assymetric', 'Asymmetric'),
        ], 'IUGR', sort=False)

    institution = fields.Many2One('gnuhealth.institution', 'Institution',
        help="Health center where this initial obstetric record was created")

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof', readonly=True,
        help="Health Professional who created this initial obstetric record")

    @classmethod
    def __setup__(cls):
        super(PatientPregnancy, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints += [
            ('gravida_uniq', Unique(t,t.name, t.gravida),
                'This pregnancy code for this patient already exists'),
        ]

        cls._error_messages.update({
            'patient_already_pregnant': 'Our records indicate that the patient'
                ' is already pregnant !'})

    @classmethod
    def validate(cls, pregnancies):
        super(PatientPregnancy, cls).validate(pregnancies)
        for pregnancy in pregnancies:
            pregnancy.check_patient_current_pregnancy()

    def check_patient_current_pregnancy(self):
        ''' Check for only one current pregnancy in the patient '''
        pregnancy = Table('gnuhealth_patient_pregnancy')
        cursor = Transaction().connection.cursor()
        patient_id = int(self.name.id)
        
        cursor.execute (*pregnancy.select(Count(pregnancy.name),
            where=(pregnancy.current_pregnancy == 'true') &
            (pregnancy.name == patient_id))) 
                                       
        records = cursor.fetchone()[0]
        if records > 1:
            self.raise_user_error('patient_already_pregnant')

    @staticmethod
    def default_current_pregnancy():
        return True

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        return HealthProf.get_health_professional()

    @fields.depends('reverse_weeks', 'pregnancy_end_date')
    def on_change_with_lmp(self):
        # Calculate the estimate on Last Menstrual Period
        # using the reverse input method, taking the
        # end of pregnancy date and number of weeks

        if (self.reverse_weeks and self.pregnancy_end_date):
            estimated_lmp = datetime.datetime.date(self.pregnancy_end_date - 
                datetime.timedelta(self.reverse_weeks*7))
                        
            return estimated_lmp 

    def get_pregnancy_data(self, name):
        """ Calculate the Pregnancy Due Date and the Number of
        weeks at the end of pregnancy when using the Last Menstrual
        Period parameter. 
        It's not calculated when using the reverse input method
        """
        if name == 'pdd':
                return self.lmp + datetime.timedelta(days=280)
                
        if name == 'pregnancy_end_age':
            if self.pregnancy_end_date:
                gestational_age = datetime.datetime.date(
                    self.pregnancy_end_date) - self.lmp
                return (gestational_age.days) / 7
            else:
                return 0

    def get_warn_icon(self, name):
        if self.warning:
            return 'gnuhealth-warning'

class PrenatalEvaluation(ModelSQL, ModelView):
    'Prenatal and Antenatal Evaluations'
    __name__ = 'gnuhealth.patient.prenatal.evaluation'

    name = fields.Many2One('gnuhealth.patient.pregnancy', 'Patient Pregnancy')
    evaluation = fields.Many2One('gnuhealth.patient.evaluation',
        'Patient Evaluation', readonly=True)
    evaluation_date = fields.DateTime('Date', required=True)
    gestational_weeks = fields.Function(fields.Integer('Gestational Weeks'),
        'get_patient_evaluation_data')
    gestational_days = fields.Function(fields.Integer('Gestational days'),
        'get_patient_evaluation_data')
    hypertension = fields.Boolean('Hypertension', help='Check this box if the'
        ' mother has hypertension')
    preeclampsia = fields.Boolean('Preeclampsia', help='Check this box if the'
        ' mother has pre-eclampsia')
    overweight = fields.Boolean('Overweight', help='Check this box if the'
        ' mother is overweight or obesity')
    diabetes = fields.Boolean('Diabetes', help='Check this box if the mother'
        ' has glucose intolerance or diabetes')
    invasive_placentation = fields.Selection([
        (None, ''),
        ('normal', 'Normal decidua'),
        ('accreta', 'Accreta'),
        ('increta', 'Increta'),
        ('percreta', 'Percreta'),
        ], 'Placentation', sort=False)
    placenta_previa = fields.Boolean('Placenta Previa')
    vasa_previa = fields.Boolean('Vasa Previa')
    fundal_height = fields.Integer('Fundal Height',
        help="Distance between the symphysis pubis and the uterine fundus "
        "(S-FD) in cm")
    fetus_heart_rate = fields.Integer('Fetus heart rate', help='Fetus heart'
        ' rate')
    efw = fields.Integer('EFW', help="Estimated Fetal Weight")
    fetal_bpd = fields.Integer('BPD', help="Fetal Biparietal Diameter")
    fetal_ac = fields.Integer('AC', help="Fetal Abdominal Circumference")
    fetal_hc = fields.Integer('HC', help="Fetal Head Circumference")
    fetal_fl = fields.Integer('FL', help="Fetal Femur Length")
    oligohydramnios = fields.Boolean('Oligohydramnios')
    polihydramnios = fields.Boolean('Polihydramnios')
    iugr = fields.Boolean('IUGR', help="Intra Uterine Growth Restriction")

    institution = fields.Many2One('gnuhealth.institution', 'Institution')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof', readonly=True,
        help="Health Professional in charge, or that who entered the \
            information in the system")

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        return HealthProf.get_health_professional()


    def get_patient_evaluation_data(self, name):
        if name == 'gestational_weeks':
            gestational_age = datetime.datetime.date(self.evaluation_date) - \
                self.name.lmp
            return (gestational_age.days) / 7
        if name == 'gestational_days':
            gestational_age = datetime.datetime.date(self.evaluation_date) - \
                self.name.lmp
            return gestational_age.days


class PuerperiumMonitor(ModelSQL, ModelView):
    'Puerperium Monitor'
    __name__ = 'gnuhealth.puerperium.monitor'

    name = fields.Many2One('gnuhealth.patient.pregnancy', 'Patient Pregnancy')
    date = fields.DateTime('Date and Time', required=True)
    # Deprecated in 1.6.4 All the clinical information will be taken at the
    # main evaluation.
    # systolic / diastolic / frequency / temperature
    systolic = fields.Integer('Systolic Pressure')
    diastolic = fields.Integer('Diastolic Pressure')
    frequency = fields.Integer('Heart Frequency')
    temperature = fields.Float('Temperature')
    lochia_amount = fields.Selection([
        (None, ''),
        ('n', 'normal'),
        ('e', 'abundant'),
        ('h', 'hemorrhage'),
        ], 'Lochia amount', sort=False)
    lochia_color = fields.Selection([
        (None, ''),
        ('r', 'rubra'),
        ('s', 'serosa'),
        ('a', 'alba'),
        ], 'Lochia color', sort=False)
    lochia_odor = fields.Selection([
        (None, ''),
        ('n', 'normal'),
        ('o', 'offensive'),
        ], 'Lochia odor', sort=False)
    uterus_involution = fields.Integer('Fundal Height',
        help="Distance between the symphysis pubis and the uterine fundus "
        "(S-FD) in cm")

    institution = fields.Many2One('gnuhealth.institution', 'Institution')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof', readonly=True,
        help="Health Professional in charge, or that who entered the \
            information in the system")

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        return HealthProf.get_health_professional()


class Perinatal(ModelSQL, ModelView):
    'Perinatal Information'
    __name__ = 'gnuhealth.perinatal'

    name = fields.Many2One('gnuhealth.patient.pregnancy', 'Patient Pregnancy')
    admission_code = fields.Char('Code')
    # 1.6.4 Gravida number and abortion information go now in the pregnancy
    # header. It will be calculated as a function if needed
    gravida_number = fields.Integer('Gravida #')
    # Deprecated. Use End of Pregnancy as a general concept in the pregnancy
    # model
    # Abortion / Stillbirth / Live Birth
    abortion = fields.Boolean('Abortion')
    stillbirth = fields.Boolean('Stillbirth')
    admission_date = fields.DateTime('Admission',
        help="Date when she was admitted to give birth", required=True)
    # Prenatal evaluations deprecated in 1.6.4. Will be computed automatically
    prenatal_evaluations = fields.Integer('Prenatal evaluations',
        help="Number of visits to the doctor during pregnancy")
    start_labor_mode = fields.Selection([
        (None, ''),
        ('v', 'Vaginal - Spontaneous'),
        ('ve', 'Vaginal - Vacuum Extraction'),
        ('vf', 'Vaginal - Forceps Extraction'),
        ('c', 'C-section'),
        ], 'Delivery mode', sort=False)
    gestational_weeks = fields.Function(fields.Integer('Gestational wks'),
        'get_perinatal_information')
    gestational_days = fields.Integer('Days')
    fetus_presentation = fields.Selection([
        (None, ''),
        ('cephalic', 'Cephalic'),
        ('breech', 'Breech'),
        ('shoulder', 'Shoulder'),
        ], 'Fetus Presentation', sort=False)
    dystocia = fields.Boolean('Dystocia')
    placenta_incomplete = fields.Boolean('Incomplete',
        help='Incomplete Placenta')
    placenta_retained = fields.Boolean('Retained', help='Retained Placenta')
    abruptio_placentae = fields.Boolean('Abruptio Placentae',
        help='Abruptio Placentae')
    episiotomy = fields.Boolean('Episiotomy')
    #Vaginal tearing and forceps variables are deprecated in 1.6.4.
    #They are included in laceration and delivery mode respectively
    vaginal_tearing = fields.Boolean('Vaginal tearing')
    forceps = fields.Boolean('Forceps')
    monitoring = fields.One2Many('gnuhealth.perinatal.monitor', 'name',
        'Monitors')
    laceration = fields.Selection([
        (None, ''),
        ('perineal', 'Perineal'),
        ('vaginal', 'Vaginal'),
        ('cervical', 'Cervical'),
        ('broad_ligament', 'Broad Ligament'),
        ('vulvar', 'Vulvar'),
        ('rectal', 'Rectal'),
        ('bladder', 'Bladder'),
        ('urethral', 'Urethral'),
        ], 'Lacerations', sort=False)
    hematoma = fields.Selection([
        (None, ''),
        ('vaginal', 'Vaginal'),
        ('vulvar', 'Vulvar'),
        ('retroperitoneal', 'Retroperitoneal'),
        ], 'Hematoma', sort=False)
    notes = fields.Text('Notes')

    institution = fields.Many2One('gnuhealth.institution', 'Institution')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof', readonly=True,
        help="Health Professional in charge, or that who entered the \
            information in the system")

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        return HealthProf.get_health_professional()




    def get_perinatal_information(self, name):
        if name == 'gestational_weeks':
            gestational_age = datetime.datetime.date(self.admission_date) - \
                self.name.lmp
            return (gestational_age.days) / 7


class PerinatalMonitor(ModelSQL, ModelView):
    'Perinatal Monitor'
    __name__ = 'gnuhealth.perinatal.monitor'

    name = fields.Many2One('gnuhealth.perinatal',
        'Patient Perinatal Evaluation')
    date = fields.DateTime('Date and Time')
    systolic = fields.Integer('Systolic Pressure')
    diastolic = fields.Integer('Diastolic Pressure')
    contractions = fields.Integer('Contractions')
    frequency = fields.Integer('Mother\'s Heart Frequency')
    dilation = fields.Integer('Cervix dilation')
    f_frequency = fields.Integer('Fetus Heart Frequency')
    meconium = fields.Boolean('Meconium')
    bleeding = fields.Boolean('Bleeding')
    fundal_height = fields.Integer('Fundal Height')
    fetus_position = fields.Selection([
        (None, ''),
        ('o', 'Occiput / Cephalic Posterior'),
        ('fb', 'Frank Breech'),
        ('cb', 'Complete Breech'),
        ('t', 'Transverse Lie'),
        ('t', 'Footling Breech'),
        ], 'Fetus Position', sort=False)


class GnuHealthPatient(ModelSQL, ModelView):
    'Add to the Medical patient_data class (gnuhealth.patient) the ' \
    'gynecological and obstetric fields.'
    __name__ = 'gnuhealth.patient'

    currently_pregnant = fields.Function(fields.Boolean('Pregnant'),
        'get_pregnancy_info')
    fertile = fields.Boolean('Fertile',
        help="Check if patient is in fertile age")
    menarche = fields.Integer('Menarche age')
    menopausal = fields.Boolean('Menopausal')
    menopause = fields.Integer('Menopause age')
    mammography = fields.Boolean('Mammography',
        help="Check if the patient does periodic mammographys")
    mammography_last = fields.Date('Last mammography',
        help="Enter the date of the last mammography")
    breast_self_examination = fields.Boolean('Breast self-examination',
        help="Check if patient does and knows how to self examine her breasts")
    pap_test = fields.Boolean('PAP test',
        help="Check if patient does periodic cytologic pelvic smear screening")
    pap_test_last = fields.Date('Last PAP test',
        help="Enter the date of the last Papanicolau test")
    colposcopy = fields.Boolean('Colposcopy',
        help="Check if the patient has done a colposcopy exam")
    colposcopy_last = fields.Date('Last colposcopy',
        help="Enter the date of the last colposcopy")
    #From version 2.6 Gravida, premature, abortions and stillbirths are now
    # functional fields, computed from the obstetric information
    gravida = fields.Function(fields.Integer('Pregnancies',
        help="Number of pregnancies, computed from Obstetric history"),
        'patient_obstetric_info')
    premature = fields.Function(fields.Integer('Premature',
     help="Preterm < 37 wks live births"),'patient_obstetric_info')
     
    abortions = fields.Function(fields.Integer('Abortions'),
        'patient_obstetric_info')
    stillbirths = fields.Function(fields.Integer('Stillbirths'),
        'patient_obstetric_info')

    full_term = fields.Integer('Full Term', help="Full term pregnancies")
    # GPA Deprecated in 1.6.4. It will be used as a function or report from the
    # other fields
    #    gpa = fields.Char('GPA',
    #        help="Gravida, Para, Abortus Notation. For example G4P3A1 : 4 " \
    #        "Pregnancies, 3 viable and 1 abortion")
    # Deprecated. The born alive number will be calculated from pregnancies -
    # abortions - stillbirths
    #    born_alive = fields.Integer('Born Alive')
    # Deceased in 1st week or after 2nd weeks are deprecated since 1.6.4 .
    # The information will be retrieved from the neonatal or infant record
    #    deaths_1st_week = fields.Integer('Deceased during 1st week',
    #        help="Number of babies that die in the first week")
    #    deaths_2nd_week = fields.Integer('Deceased after 2nd week',
    #        help="Number of babies that die after the second week")
    # Perinatal Deprecated since 1.6.4 - Included in the obstetric history
    #    perinatal = fields.One2Many('gnuhealth.perinatal', 'name',
    #        'Perinatal Info')
    menstrual_history = fields.One2Many('gnuhealth.patient.menstrual_history',
        'name', 'Menstrual History')
    mammography_history = fields.One2Many(
        'gnuhealth.patient.mammography_history', 'name', 'Mammography History',
         states={'invisible': Not(Bool(Eval('mammography')))},
        )
    pap_history = fields.One2Many('gnuhealth.patient.pap_history', 'name',
        'PAP smear History',
         states={'invisible': Not(Bool(Eval('pap_test')))},
        )
    colposcopy_history = fields.One2Many(
        'gnuhealth.patient.colposcopy_history', 'name', 'Colposcopy History',
         states={'invisible': Not(Bool(Eval('colposcopy')))},
        )
    pregnancy_history = fields.One2Many('gnuhealth.patient.pregnancy', 'name',
        'Pregnancies')

    def get_pregnancy_info(self, name):
        if name == 'currently_pregnant':
            for pregnancy_history in self.pregnancy_history:
                if pregnancy_history.current_pregnancy:
                    return True
        return False

    def patient_obstetric_info(self,name):
        ''' Return the number of pregnancies, perterm, 
        abortion and stillbirths '''

        counter=0
        pregnancies = len(self.pregnancy_history)
         
        if (name == "gravida"):
            return pregnancies 

        if (name == "premature"):
            prematures=0
            while counter < pregnancies:
                result = self.pregnancy_history[counter].pregnancy_end_result
                preg_weeks = self.pregnancy_history[counter].pregnancy_end_age
                if (result == "live_birth" and
                    preg_weeks < 37):
                        prematures=prematures+1
                counter=counter+1
            return prematures

        if (name == "abortions"):
            abortions=0
            while counter < pregnancies:
                result = self.pregnancy_history[counter].pregnancy_end_result
                preg_weeks = self.pregnancy_history[counter].pregnancy_end_age
                if (result == "abortion"):
                    abortions=abortions+1
                counter=counter+1

            return abortions

        if (name == "stillbirths"):
            stillbirths=0
            while counter < pregnancies:
                result = self.pregnancy_history[counter].pregnancy_end_result
                preg_weeks = self.pregnancy_history[counter].pregnancy_end_age
                if (result == "stillbirth"):
                    stillbirths=stillbirths+1
                counter=counter+1
            return stillbirths


    @classmethod
    def view_attributes(cls):
        return [('//page[@id="page_gyneco_obs"]', 'states', {
                'invisible': Equal(Eval('biological_sex'), 'm'),
                })]

class PatientMenstrualHistory(ModelSQL, ModelView):
    'Menstrual History'
    __name__ = 'gnuhealth.patient.menstrual_history'

    name = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True,
        required=True)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation', 'Evaluation',
        domain=[('patient', '=', Eval('name'))],
        depends=['name'])
    evaluation_date = fields.Date('Date', help="Evaluation Date",
        required=True)
    lmp = fields.Date('LMP', help="Last Menstrual Period", required=True)
    lmp_length = fields.Integer('Length', required=True)
    is_regular = fields.Boolean('Regular')
    dysmenorrhea = fields.Boolean('Dysmenorrhea')
    frequency = fields.Selection([
        ('amenorrhea', 'amenorrhea'),
        ('oligomenorrhea', 'oligomenorrhea'),
        ('eumenorrhea', 'eumenorrhea'),
        ('polymenorrhea', 'polymenorrhea'),
        ], 'frequency', sort=False)
    volume = fields.Selection([
        ('hypomenorrhea', 'hypomenorrhea'),
        ('normal', 'normal'),
        ('menorrhagia', 'menorrhagia'),
        ], 'volume', sort=False)

    institution = fields.Many2One('gnuhealth.institution', 'Institution')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Reviewed', readonly=True,
        help="Health Professional who reviewed the information")

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        return HealthProf.get_health_professional()


    @staticmethod
    def default_evaluation_date():
        return Pool().get('ir.date').today()

    @staticmethod
    def default_evaluation_date():
        return Pool().get('ir.date').today()

    @staticmethod
    def default_frequency():
        return 'eumenorrhea'

    @staticmethod
    def default_volume():
        return 'normal'


class PatientMammographyHistory(ModelSQL, ModelView):
    'Mammography History'
    __name__ = 'gnuhealth.patient.mammography_history'

    name = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True,
        required=True)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation', 'Evaluation',
        domain=[('patient', '=', Eval('name'))],
        depends=['name'])
    evaluation_date = fields.Date('Date', help="Date", required=True)
    last_mammography = fields.Date('Previous', help="Last Mammography")
    result = fields.Selection([
        (None, ''),
        ('normal', 'normal'),
        ('abnormal', 'abnormal'),
        ], 'result', help="Please check the lab test results if the module is \
            installed", sort=False)
    comments = fields.Char('Remarks')

    institution = fields.Many2One('gnuhealth.institution', 'Institution')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Reviewed', readonly=True,
        help="Health Professional who last reviewed the test")

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        return HealthProf.get_health_professional()


    @staticmethod
    def default_evaluation_date():
        return Pool().get('ir.date').today()

    @staticmethod
    def default_last_mammography():
        return Pool().get('ir.date').today()


class PatientPAPHistory(ModelSQL, ModelView):
    'PAP Test History'
    __name__ = 'gnuhealth.patient.pap_history'

    name = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True,
        required=True)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation', 'Evaluation',
        domain=[('patient', '=', Eval('name'))],
        depends=['name'])
    evaluation_date = fields.Date('Date', help="Date", required=True)
    last_pap = fields.Date('Previous', help="Last Papanicolau")
    result = fields.Selection([
        (None, ''),
        ('negative', 'Negative'),
        ('c1', 'ASC-US'),
        ('c2', 'ASC-H'),
        ('g1', 'ASG'),
        ('c3', 'LSIL'),
        ('c4', 'HSIL'),
        ('g4', 'AIS'),
        ], 'result', help="Please check the lab results if the module is \
            installed", sort=False)
    comments = fields.Char('Remarks')

    institution = fields.Many2One('gnuhealth.institution', 'Institution')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Reviewed', readonly=True,
        help="Health Professional who last reviewed the test")

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        return HealthProf.get_health_professional()


    @staticmethod
    def default_evaluation_date():
        return Pool().get('ir.date').today()

    @staticmethod
    def default_last_pap():
        return Pool().get('ir.date').today()


class PatientColposcopyHistory(ModelSQL, ModelView):
    'Colposcopy History'
    __name__ = 'gnuhealth.patient.colposcopy_history'

    name = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True,
        required=True)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation', 'Evaluation',
        domain=[('patient', '=', Eval('name'))],
        depends=['name'])
    evaluation_date = fields.Date('Date', help="Date", required=True)
    last_colposcopy = fields.Date('Previous', help="Last colposcopy")
    result = fields.Selection([
        (None, ''),
        ('normal', 'normal'),
        ('abnormal', 'abnormal'),
        ], 'result', help="Please check the lab test results if the module is \
            installed", sort=False)
    comments = fields.Char('Remarks')

    institution = fields.Many2One('gnuhealth.institution', 'Institution')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Reviewed', readonly=True,
        help="Health Professional who last reviewed the test")

    @staticmethod
    def default_institution():
        HealthInst = Pool().get('gnuhealth.institution')
        institution = HealthInst.get_institution()
        return institution

    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        return HealthProf.get_health_professional()


    @staticmethod
    def default_evaluation_date():
        return Pool().get('ir.date').today()

    @staticmethod
    def default_last_colposcopy():
        return Pool().get('ir.date').today()



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#    Copyright (C) 2013  Sebastian Marro <smarro@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from trytond.wizard import Wizard, StateView, Button, StateTransition
from trytond.model import ModelView
from trytond.transaction import Transaction
from trytond.pool import Pool

__all__ = ['CreatePrescriptionStockMoveInit','CreatePrescriptionStockMove']

class CreatePrescriptionStockMoveInit(ModelView):
    'Create Prescription Stock Move Init'
    __name__ = 'gnuhealth.prescription.stock.move.init'


class CreatePrescriptionStockMove(Wizard):
    'Create Prescription Stock Move'
    __name__ = 'gnuhealth.prescription.stock.move.create'

    start = StateView('gnuhealth.prescription.stock.move.init',
            'health_stock.view_create_prescription_stock_move', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Create Stock Move', 'create_stock_move',
                'tryton-ok', True),
            ])
    create_stock_move = StateTransition()

    def transition_create_stock_move(self):
        pool = Pool()
        StockMove = pool.get('stock.move')
        Prescription = pool.get('gnuhealth.prescription.order')

        moves = []
        prescriptions = Prescription.browse(Transaction().context.get(
            'active_ids'))
        for prescription in prescriptions:

            if prescription.moves:
                self.raise_user_error('stock_move_exists')

            if not prescription.pharmacy:
                self.raise_user_error('no_pharmacy_selected')

            from_location = prescription.pharmacy.warehouse
            if from_location.type == 'warehouse':
                from_location = from_location.storage_location
            to_location = prescription.patient.name.customer_location

            for line in prescription.prescription_line:
                move = StockMove()
                move.origin = prescription
                move.from_location = from_location
                move.to_location = to_location
                move.product = line.medicament.name
                move.unit_price = line.medicament.name.list_price
                move.quantity = line.quantity
                move.uom = line.medicament.name.default_uom
                moves.append(move)
        StockMove.save(moves)
        StockMove.do(moves)
        return 'end'

    @classmethod
    def __setup__(cls):
        super(CreatePrescriptionStockMove, cls).__setup__()
        cls._error_messages.update({
            'stock_move_exists':
                'Stock moves already exists!.',
            'no_pharmacy_selected':
                'You need to select a pharmacy.',
            })




# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import tryton.rpc as rpc
from tryton.common import RPCExecute, warning, message
from tryton.gui.window.form import Form
import gettext
import numpy as np
import pygtk
pygtk.require('2.0')
import gtk
from datetime import datetime
import cv2


_ = gettext.gettext



def set_attachment(data, frame):
    #Store the frame in a container
    rc, container = cv2.imencode(".png",frame)
    container = bytearray(container)
    document_model = data['model']
    ref=document_model + ',' + str(data['ids'][0])
    timestamp = str(datetime.now())
    attach_name = "GNU Health media " + timestamp

    #Store the attachment
    save_attach = rpc.execute(
        'model', 'ir.attachment', 'create',
        [{'name': attach_name,
         'type': 'data',
         'resource': ref,
         'description':'From GNU Health camera',
         'data':container,
        }], rpc.CONTEXT)
    
    if save_attach:
        msg = "Attachment saved correctly !\n\n" \
            "Please refresh the view"
        message(
            _(msg),
        )
        return True

def get_help():
    msg = "========== GNU Health Camera Help ==========\n\n" \
        "a = Attach media file in the current model\n\n" \
        "[space] = Set the picture in model (when available)\n\n" \
        "h = This help message\n\n" \
        "q = Quit the camera application\n\n" 
        
    message(
        _(msg),
    )
    return True

def set_media(data, frame):
    #Store the frame in a container
    rc, container = cv2.imencode(".png",frame)
    container = bytearray(container)
    document_model = data['model']

    target_field = None
    
    # Photo in person registration
    if (document_model == 'party.party'):
        target_field = 'photo'
        
    if (target_field):
        rpc.execute(
            'model', document_model, 'write',
            data['ids'],
            {target_field:container}, rpc.CONTEXT)
        
        return True
    else:
        return False
    
def main (data):

    # Allow only one record

    if (len(data['ids']) == 0):
        warning(
            _('Please choose one record associated to this picture / video'),
            _('You need to select a record !'),
        )
        return

    if (len(data['ids']) > 1):
        warning(
            _('Please choose only one record for this picture / video'),
            _('Multiple records selectd !'),
        )
        return

    document_model = data['model']

    # Open the webcam device
    cap = cv2.VideoCapture(0)

    preview = False
    
    while(True):
        # Grab the frames
        try:
            rc, frame = cap.read()
        except:
            cleanup()
            
        # Display the resulting frame
        
        cv2.imshow('== GNU Health Camera ==',frame)

        keypressed = cv2.waitKey(1)

        if  keypressed == ord(' '):
            
            cv2.imshow("Preview",frame)
           
            # Make a backup copy  
            cv2.imwrite('/tmp/gnuhealth_snapshot_preview.png',frame)              

            # call set media   
            set_media(data, frame)
            
            preview = True

        if  keypressed == ord('a'):
            
            cv2.imshow("Preview",frame)
            
            # Make a backup copy  
            cv2.imwrite('/tmp/gnuhealth_snapshot_preview.png',frame)              

            # call set media   
            set_attachment(data, frame)
            break            

        if keypressed == ord('q'):
            break

        if  keypressed == ord('h'):
            get_help()
            
    cleanup(cap)

    # Reload the form
    a = Form(document_model, res_id=data['ids'][0])
    a.sig_reload(test_modified=False)

def get_plugins(model):
    return [
        (_('GNU Health camera'), main),
        ]

def cleanup(cap):
    #Cleanup 
    cap.release()
    cv2.destroyAllWindows()



from StringIO import StringIO
from operator import attrgetter
from server.common import safe_attrgetter
import server.fhir as supermod
from .health_mixin import ExportXMLMixin

try:
    from flask import url_for
    RUN_FLASK=True
except:
    from .datastore import dumb_url_generate
    RUN_FLASK=False


class MedicationStatement_Map:
    """Holds essential mappings and information for
        the MedicationStatement class
    """

    root_search=[('name', '!=', None)]

    resource_search_params={
            '_id': 'token',
            'patient': 'reference',
            'when-given': 'date',
            'medication': 'reference',
            'identifier': None,
            'device': None,
            '_language': None}

    chain_map={'patient': 'Patient',
            'medication': 'Medication'}

    search_mapping={
            '_id': ['id'],
            'patient': ['name'],
            'when-given': ['start_treatment'],
            'medication': ['medicament']}


    url_prefixes={}
    model_mapping={'gnuhealth.patient.medication':
            {
                'patient': 'name',
                'when-given': {
                        'start': 'start_treatment',
                        'end': 'end_treatment'},
                'medication': 'medicament',
                'dose': {
                        'quantity': 'dose',
                        'units': 'dose_unit.name',
                        'route': 'route',
                        'asNeededBoolean': 'frequency_prn'}
            }}

class health_MedicationStatement(supermod.MedicationStatement, MedicationStatement_Map, ExportXMLMixin):
    def __init__(self, *args, **kwargs):
        rec = kwargs.pop('gnu_record', None)
        super(health_MedicationStatement, self).__init__(*args, **kwargs)
        if rec:
            self.set_gnu_medication_statement(rec)

    def set_gnu_medication_statement(self, medication_statement):
        """Set the GNU Health record
        ::::
            params:
                medication_statement ===> Health model
            returns:
                instance

        """
        self.medication_statement = medication_statement
        self.model_type = self.medication_statement.__name__

        # Only certain models
        if self.model_type not in self.model_mapping:
            raise ValueError('Not a valid model')

        self.map = self.model_mapping[self.model_type]

        self.__import_from_gnu_medication_statement()

    def __import_from_gnu_medication_statement(self):
        """Import the data from the model"""
        if self.medication_statement:
            self.__set_gnu_patient()
            self.__set_gnu_when_given()
            self.__set_gnu_medication()
            #self.__set_gnu_dosage()

            self.__set_feed_info()

    def __set_feed_info(self):
        """Sets the feed-relevant info"""
        if self.medication_statement:
            if RUN_FLASK:
                uri = url_for('ms_record',
                            log_id=self.medication_statement.id,
                            _external=True)
            else:
                uri = dumb_url_generate(['MedicationStatement', self.medication_statement.id])
            self.feed={'id': uri,
                    'published': self.medication_statement.create_date,
                    'updated': self.medication_statement.write_date or self.medication_statement.create_date,
                    'title': '{} for {}'.format(
                            self.medication_statement.medicament.name.name,
                            self.medication_statement.name.rec_name)
                        }

    def __set_gnu_patient(self):
        """Set the patient info"""
        if self.medication_statement:
            patient = attrgetter(self.map['patient'])(self.medication_statement)
            if RUN_FLASK:
                uri = url_for('pat_record', log_id=patient.id)
            else:
                uri = dumb_url_generate(['Patient', patient.id])
            display = patient.rec_name
            ref=supermod.ResourceReference()
            ref.display = supermod.string(value=display)
            ref.reference = supermod.string(value=uri)
            self.set_patient(ref)

    def __set_gnu_dosage(self):
        """Set dosage"""
        # TODO Add better dosage info
        # FIX On hold since many changes upcoming to the dosage
        if self.medication_statement:
            d = supermod.MedicationStatement_Dosage()


            d.quantity = supermod.Quantity()
            f = safe_attrgetter(self.medication_statement, self.map['dose']['quantity'])
            if f:
                d.quantity.value = supermod.decimal(value=f)
            u = safe_attrgetter(self.medication_statement, self.map['dose']['units'])
            if u:
                d.quantity.units = supermod.string(value=u)

            d.route = supermod.CodeableConcept()
            r = safe_attrgetter(self.medication_statement, self.map['dose']['route'])
            if r:
                d.route.coding = [supermod.Coding()]
                d.route.coding[0].code = supermod.code(value=r.code)
                d.route.coding[0].display = supermod.string(value=r.name)
                d.route.text = supermod.string(r.name)

            # PRN?
            an = safe_attrgetter(self.medication_statement,
                            self.map['dose']['asNeededBoolean'])
            if an:
                self.set_asNeededBoolean(supermod.boolean(value=True))

            d.timing = supermod.Schedule()

            if self.medication_statement.infusion:
                d.rate

    def __set_gnu_when_given(self):
        """Set datetime when given"""
        # TODO Maybe add hours, minutes, etc.
        if self.medication_statement:
            start = safe_attrgetter(self.medication_statement,
                                    self.map['when-given']['start'])
            end = safe_attrgetter(self.medication_statement,
                                    self.map['when-given']['end'])
            if start:
                p = supermod.Period()
                p.start = supermod.dateTime(value=start.strftime('%Y/%m/%d'))
                if end:
                    p.end = supermod.dateTime(value=end.strftime('%Y/%m/%d'))
                self.set_whenGiven(p)


    def __set_gnu_medication(self):
        if self.medication_statement:
            med = attrgetter(self.map['medication'])(self.medication_statement)
            if RUN_FLASK:
                uri = url_for('med_record', log_id=med.id)
            else:
                uri = dumb_url_generate(['Medication', med.id])
            display = med.active_component
            ref=supermod.ResourceReference()
            ref.display = supermod.string(value=display)
            ref.reference = supermod.string(value=uri)
            self.set_medication(ref)

supermod.MedicationStatement.subclass=health_MedicationStatement



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2013 Sebastián Marro <smarro@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
    Button
from trytond.transaction import Transaction

__all__ = ['OpenPediatricsGrowthChartsWHOReportStart',
    'OpenPediatricsGrowthChartsWHOReport']


class OpenPediatricsGrowthChartsWHOReportStart(ModelView):
    'Open Pediatrics Growth Charts WHO Report Start'
    __name__ = 'gnuhealth.pediatrics.growth.charts.who.report.open.start'

    indicator = fields.Selection([
        ('l/h-f-a', 'Length/height for age'),
        ('w-f-a', 'Weight for age'),
        ('bmi-f-a', 'Body mass index for age (BMI for age)'),
        ], 'Indicator', sort=False, required=True)
    measure = fields.Selection([
        ('p', 'percentiles'),
        ('z', 'z-scores'),
        ], 'Measure', required=True)


class OpenPediatricsGrowthChartsWHOReport(Wizard):
    'Open Pediatrics Growth Charts WHO Report'
    __name__ = 'gnuhealth.pediatrics.growth.charts.who.report.open'

    start = StateView('gnuhealth.pediatrics.growth.charts.who.report.open.start',
        'health_pediatrics_growth_charts_who.growth_charts_who_open_start_view_form', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Open', 'choose', 'tryton-ok', default=True),
            ])
    choose = StateTransition()
    print_wfa = StateAction('health_pediatrics_growth_charts_who.report_pediatrics_growth_charts_who_wfa')
    print_lhfa = StateAction('health_pediatrics_growth_charts_who.report_pediatrics_growth_charts_who_lhfa')
    print_bmifa = StateAction('health_pediatrics_growth_charts_who.report_pediatrics_growth_charts_who_bmifa')

    def transition_choose(self):
        if self.start.indicator == 'w-f-a':
            return 'print_wfa'
        elif self.start.indicator == 'l/h-f-a':
            return 'print_lhfa'
        else:
            return 'print_bmifa'

    def fill_data(self):
        return {
            'patient': Transaction().context.get('active_id'),
            'indicator': self.start.indicator,
            'measure': self.start.measure,
            }

    def do_print_wfa(self, action):
        return action, self.fill_data()

    def do_print_lhfa(self, action):
        return action, self.fill_data()

    def do_print_bmifa(self, action):
        return action, self.fill_data()



from flask_login import (login_user, UserMixin,
                            login_required, logout_user,
                            current_user, current_app)
from flask_wtf import Form
from flask import Blueprint, render_template, request, redirect, url_for, flash
from wtforms import StringField, PasswordField, SubmitField
from wtforms.validators import DataRequired
from server.common import tryton, login_manager
import trytond.security
from trytond.transaction import Transaction
import trytond

auth_endpoint = Blueprint('auth_endpoint', __name__,
                            template_folder='templates')

user_model = tryton.pool.get('res.user')

@login_manager.user_loader
def load_user(user_id):
    return User(uid=user_id)

class LoginForm(Form):
    """The login form
    """
    username = StringField('Username', [DataRequired()])
    password = PasswordField('Password', [DataRequired()])
    submit=SubmitField('Login')

@tryton.transaction()
def get_user_id(uid=None, username=None, password=None):
    """Return user info from tryton
    """
    if uid is not None:
        user=user_model.search([('id', '=', uid)])
        if user:
            return unicode(user[0].id)
    if username is not None and password is not None:
        user_id = user_model.get_login(username, password)
        if user_id:
            return unicode(user_id)
    return None

class User(UserMixin):
    """The user class
    """

    def __init__(self, uid=None, username=None, password=None):
        self.id=None

        ident = get_user_id(uid, username, password)
        if ident is not None:
            self.id = ident

    def get_id(self):
        return self.id

@auth_endpoint.route("/login", methods=["GET", "POST"])
def login():
    """Login view"""
    if current_user.is_authenticated:
        return redirect(url_for('auth_endpoint.home'))
    form = LoginForm()
    if form.validate_on_submit():
        u = User(username=request.form['username'], password=request.form['password'])
        login_user(u, remember=True)
        flash('Logged in')
        n = request.args.get('next', url_for('auth_endpoint.home'))
        return redirect(n)
    return render_template("login.html", form=form)

@auth_endpoint.route("/logout", methods=["GET"])
@login_required
def logout():
    """Logout view"""
    logout_user()
    flash('Logged out')
    return redirect(url_for('auth_endpoint.login'))

@auth_endpoint.route("/home", methods=["GET"])
@login_required
def home():
    """Home view"""
    return render_template('home.html')



from .base_classes import *
from .support_functions import *


class Specimen(Resource):
    """Sample for analysis.If the element is present, it must have either a
    @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, type_=None, source=None, subject=None, accessionIdentifier=None, receivedTime=None, collection=None, treatment=None, container=None):
        self.original_tagname_ = None
        super(Specimen, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.type_ = type_
        if source is None:
            self.source = []
        else:
            self.source = source
        self.subject = subject
        self.accessionIdentifier = accessionIdentifier
        self.receivedTime = receivedTime
        self.collection = collection
        if treatment is None:
            self.treatment = []
        else:
            self.treatment = treatment
        if container is None:
            self.container = []
        else:
            self.container = container
    def factory(*args_, **kwargs_):
        if Specimen.subclass:
            return Specimen.subclass(*args_, **kwargs_)
        else:
            return Specimen(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_source(self): return self.source
    def set_source(self, source): self.source = source
    def add_source(self, value): self.source.append(value)
    def insert_source(self, index, value): self.source[index] = value
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_accessionIdentifier(self): return self.accessionIdentifier
    def set_accessionIdentifier(self, accessionIdentifier): self.accessionIdentifier = accessionIdentifier
    def get_receivedTime(self): return self.receivedTime
    def set_receivedTime(self, receivedTime): self.receivedTime = receivedTime
    def get_collection(self): return self.collection
    def set_collection(self, collection): self.collection = collection
    def get_treatment(self): return self.treatment
    def set_treatment(self, treatment): self.treatment = treatment
    def add_treatment(self, value): self.treatment.append(value)
    def insert_treatment(self, index, value): self.treatment[index] = value
    def get_container(self): return self.container
    def set_container(self, container): self.container = container
    def add_container(self, value): self.container.append(value)
    def insert_container(self, index, value): self.container[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.type_ is not None or
            self.source or
            self.subject is not None or
            self.accessionIdentifier is not None or
            self.receivedTime is not None or
            self.collection is not None or
            self.treatment or
            self.container or
            super(Specimen, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Specimen', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Specimen', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Specimen'):
        super(Specimen, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen')
    def exportChildren(self, outfile, level, namespace_='', name_='Specimen', fromsubclass_=False, pretty_print=True):
        super(Specimen, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        for source_ in self.source:
            source_.export(outfile, level, namespace_, name_='source', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.accessionIdentifier is not None:
            self.accessionIdentifier.export(outfile, level, namespace_, name_='accessionIdentifier', pretty_print=pretty_print)
        if self.receivedTime is not None:
            self.receivedTime.export(outfile, level, namespace_, name_='receivedTime', pretty_print=pretty_print)
        if self.collection is not None:
            self.collection.export(outfile, level, namespace_, name_='collection', pretty_print=pretty_print)
        for treatment_ in self.treatment:
            treatment_.export(outfile, level, namespace_, name_='treatment', pretty_print=pretty_print)
        for container_ in self.container:
            container_.export(outfile, level, namespace_, name_='container', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Specimen', mapping_=None):
        element = super(Specimen, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        for source_ in self.source:
            source_.to_etree(element, name_='source', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.accessionIdentifier is not None:
            accessionIdentifier_ = self.accessionIdentifier
            accessionIdentifier_.to_etree(element, name_='accessionIdentifier', mapping_=mapping_)
        if self.receivedTime is not None:
            receivedTime_ = self.receivedTime
            receivedTime_.to_etree(element, name_='receivedTime', mapping_=mapping_)
        if self.collection is not None:
            collection_ = self.collection
            collection_.to_etree(element, name_='collection', mapping_=mapping_)
        for treatment_ in self.treatment:
            treatment_.to_etree(element, name_='treatment', mapping_=mapping_)
        for container_ in self.container:
            container_.to_etree(element, name_='container', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Specimen, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'source':
            obj_ = Specimen_Source.factory()
            obj_.build(child_)
            self.source.append(obj_)
            obj_.original_tagname_ = 'source'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'accessionIdentifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.accessionIdentifier = obj_
            obj_.original_tagname_ = 'accessionIdentifier'
        elif nodeName_ == 'receivedTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.receivedTime = obj_
            obj_.original_tagname_ = 'receivedTime'
        elif nodeName_ == 'collection':
            obj_ = Specimen_Collection.factory()
            obj_.build(child_)
            self.collection = obj_
            obj_.original_tagname_ = 'collection'
        elif nodeName_ == 'treatment':
            obj_ = Specimen_Treatment.factory()
            obj_.build(child_)
            self.treatment.append(obj_)
            obj_.original_tagname_ = 'treatment'
        elif nodeName_ == 'container':
            obj_ = Specimen_Container.factory()
            obj_.build(child_)
            self.container.append(obj_)
            obj_.original_tagname_ = 'container'
        super(Specimen, self).buildChildren(child_, node, nodeName_, True)
# end class Specimen


class Specimen_Source(BackboneElement):
    """Sample for analysis."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, relationship=None, target=None):
        self.original_tagname_ = None
        super(Specimen_Source, self).__init__(id, extension, modifierExtension, )
        self.relationship = relationship
        if target is None:
            self.target = []
        else:
            self.target = target
    def factory(*args_, **kwargs_):
        if Specimen_Source.subclass:
            return Specimen_Source.subclass(*args_, **kwargs_)
        else:
            return Specimen_Source(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_relationship(self): return self.relationship
    def set_relationship(self, relationship): self.relationship = relationship
    def get_target(self): return self.target
    def set_target(self, target): self.target = target
    def add_target(self, value): self.target.append(value)
    def insert_target(self, index, value): self.target[index] = value
    def hasContent_(self):
        if (
            self.relationship is not None or
            self.target or
            super(Specimen_Source, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Specimen.Source', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen.Source')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Specimen.Source', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Specimen.Source'):
        super(Specimen_Source, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen.Source')
    def exportChildren(self, outfile, level, namespace_='', name_='Specimen.Source', fromsubclass_=False, pretty_print=True):
        super(Specimen_Source, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.relationship is not None:
            self.relationship.export(outfile, level, namespace_, name_='relationship', pretty_print=pretty_print)
        for target_ in self.target:
            target_.export(outfile, level, namespace_, name_='target', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Specimen.Source', mapping_=None):
        element = super(Specimen_Source, self).to_etree(parent_element, name_, mapping_)
        if self.relationship is not None:
            relationship_ = self.relationship
            relationship_.to_etree(element, name_='relationship', mapping_=mapping_)
        for target_ in self.target:
            target_.to_etree(element, name_='target', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Specimen_Source, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'relationship':
            obj_ = HierarchicalRelationshipType.factory()
            obj_.build(child_)
            self.relationship = obj_
            obj_.original_tagname_ = 'relationship'
        elif nodeName_ == 'target':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.target.append(obj_)
            obj_.original_tagname_ = 'target'
        super(Specimen_Source, self).buildChildren(child_, node, nodeName_, True)
# end class Specimen_Source


class Specimen_Collection(BackboneElement):
    """Sample for analysis.Time when specimen was collected from subject -
    the physiologically relevant time."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, collector=None, comment=None, collectedDateTime=None, collectedPeriod=None, quantity=None, method=None, sourceSite=None):
        self.original_tagname_ = None
        super(Specimen_Collection, self).__init__(id, extension, modifierExtension, )
        self.collector = collector
        if comment is None:
            self.comment = []
        else:
            self.comment = comment
        self.collectedDateTime = collectedDateTime
        self.collectedPeriod = collectedPeriod
        self.quantity = quantity
        self.method = method
        self.sourceSite = sourceSite
    def factory(*args_, **kwargs_):
        if Specimen_Collection.subclass:
            return Specimen_Collection.subclass(*args_, **kwargs_)
        else:
            return Specimen_Collection(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_collector(self): return self.collector
    def set_collector(self, collector): self.collector = collector
    def get_comment(self): return self.comment
    def set_comment(self, comment): self.comment = comment
    def add_comment(self, value): self.comment.append(value)
    def insert_comment(self, index, value): self.comment[index] = value
    def get_collectedDateTime(self): return self.collectedDateTime
    def set_collectedDateTime(self, collectedDateTime): self.collectedDateTime = collectedDateTime
    def get_collectedPeriod(self): return self.collectedPeriod
    def set_collectedPeriod(self, collectedPeriod): self.collectedPeriod = collectedPeriod
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def get_method(self): return self.method
    def set_method(self, method): self.method = method
    def get_sourceSite(self): return self.sourceSite
    def set_sourceSite(self, sourceSite): self.sourceSite = sourceSite
    def hasContent_(self):
        if (
            self.collector is not None or
            self.comment or
            self.collectedDateTime is not None or
            self.collectedPeriod is not None or
            self.quantity is not None or
            self.method is not None or
            self.sourceSite is not None or
            super(Specimen_Collection, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Specimen.Collection', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen.Collection')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Specimen.Collection', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Specimen.Collection'):
        super(Specimen_Collection, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen.Collection')
    def exportChildren(self, outfile, level, namespace_='', name_='Specimen.Collection', fromsubclass_=False, pretty_print=True):
        super(Specimen_Collection, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.collector is not None:
            self.collector.export(outfile, level, namespace_, name_='collector', pretty_print=pretty_print)
        for comment_ in self.comment:
            comment_.export(outfile, level, namespace_, name_='comment', pretty_print=pretty_print)
        if self.collectedDateTime is not None:
            self.collectedDateTime.export(outfile, level, namespace_, name_='collectedDateTime', pretty_print=pretty_print)
        if self.collectedPeriod is not None:
            self.collectedPeriod.export(outfile, level, namespace_, name_='collectedPeriod', pretty_print=pretty_print)
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
        if self.method is not None:
            self.method.export(outfile, level, namespace_, name_='method', pretty_print=pretty_print)
        if self.sourceSite is not None:
            self.sourceSite.export(outfile, level, namespace_, name_='sourceSite', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Specimen.Collection', mapping_=None):
        element = super(Specimen_Collection, self).to_etree(parent_element, name_, mapping_)
        if self.collector is not None:
            collector_ = self.collector
            collector_.to_etree(element, name_='collector', mapping_=mapping_)
        for comment_ in self.comment:
            comment_.to_etree(element, name_='comment', mapping_=mapping_)
        if self.collectedDateTime is not None:
            collectedDateTime_ = self.collectedDateTime
            collectedDateTime_.to_etree(element, name_='collectedDateTime', mapping_=mapping_)
        if self.collectedPeriod is not None:
            collectedPeriod_ = self.collectedPeriod
            collectedPeriod_.to_etree(element, name_='collectedPeriod', mapping_=mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        if self.method is not None:
            method_ = self.method
            method_.to_etree(element, name_='method', mapping_=mapping_)
        if self.sourceSite is not None:
            sourceSite_ = self.sourceSite
            sourceSite_.to_etree(element, name_='sourceSite', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Specimen_Collection, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'collector':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.collector = obj_
            obj_.original_tagname_ = 'collector'
        elif nodeName_ == 'comment':
            obj_ = string.factory()
            obj_.build(child_)
            self.comment.append(obj_)
            obj_.original_tagname_ = 'comment'
        elif nodeName_ == 'collectedDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.collectedDateTime = obj_
            obj_.original_tagname_ = 'collectedDateTime'
        elif nodeName_ == 'collectedPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.collectedPeriod = obj_
            obj_.original_tagname_ = 'collectedPeriod'
        elif nodeName_ == 'quantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        elif nodeName_ == 'method':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.method = obj_
            obj_.original_tagname_ = 'method'
        elif nodeName_ == 'sourceSite':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.sourceSite = obj_
            obj_.original_tagname_ = 'sourceSite'
        super(Specimen_Collection, self).buildChildren(child_, node, nodeName_, True)
# end class Specimen_Collection


class Specimen_Treatment(BackboneElement):
    """Sample for analysis."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, description=None, procedure=None, additive=None):
        self.original_tagname_ = None
        super(Specimen_Treatment, self).__init__(id, extension, modifierExtension, )
        self.description = description
        self.procedure = procedure
        if additive is None:
            self.additive = []
        else:
            self.additive = additive
    def factory(*args_, **kwargs_):
        if Specimen_Treatment.subclass:
            return Specimen_Treatment.subclass(*args_, **kwargs_)
        else:
            return Specimen_Treatment(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_procedure(self): return self.procedure
    def set_procedure(self, procedure): self.procedure = procedure
    def get_additive(self): return self.additive
    def set_additive(self, additive): self.additive = additive
    def add_additive(self, value): self.additive.append(value)
    def insert_additive(self, index, value): self.additive[index] = value
    def hasContent_(self):
        if (
            self.description is not None or
            self.procedure is not None or
            self.additive or
            super(Specimen_Treatment, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Specimen.Treatment', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen.Treatment')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Specimen.Treatment', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Specimen.Treatment'):
        super(Specimen_Treatment, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen.Treatment')
    def exportChildren(self, outfile, level, namespace_='', name_='Specimen.Treatment', fromsubclass_=False, pretty_print=True):
        super(Specimen_Treatment, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.procedure is not None:
            self.procedure.export(outfile, level, namespace_, name_='procedure', pretty_print=pretty_print)
        for additive_ in self.additive:
            additive_.export(outfile, level, namespace_, name_='additive', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Specimen.Treatment', mapping_=None):
        element = super(Specimen_Treatment, self).to_etree(parent_element, name_, mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.procedure is not None:
            procedure_ = self.procedure
            procedure_.to_etree(element, name_='procedure', mapping_=mapping_)
        for additive_ in self.additive:
            additive_.to_etree(element, name_='additive', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Specimen_Treatment, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'procedure':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.procedure = obj_
            obj_.original_tagname_ = 'procedure'
        elif nodeName_ == 'additive':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.additive.append(obj_)
            obj_.original_tagname_ = 'additive'
        super(Specimen_Treatment, self).buildChildren(child_, node, nodeName_, True)
# end class Specimen_Treatment


class Specimen_Container(BackboneElement):
    """Sample for analysis."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, identifier=None, description=None, type_=None, capacity=None, specimenQuantity=None, additive=None):
        self.original_tagname_ = None
        super(Specimen_Container, self).__init__(id, extension, modifierExtension, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.description = description
        self.type_ = type_
        self.capacity = capacity
        self.specimenQuantity = specimenQuantity
        self.additive = additive
    def factory(*args_, **kwargs_):
        if Specimen_Container.subclass:
            return Specimen_Container.subclass(*args_, **kwargs_)
        else:
            return Specimen_Container(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_capacity(self): return self.capacity
    def set_capacity(self, capacity): self.capacity = capacity
    def get_specimenQuantity(self): return self.specimenQuantity
    def set_specimenQuantity(self, specimenQuantity): self.specimenQuantity = specimenQuantity
    def get_additive(self): return self.additive
    def set_additive(self, additive): self.additive = additive
    def hasContent_(self):
        if (
            self.identifier or
            self.description is not None or
            self.type_ is not None or
            self.capacity is not None or
            self.specimenQuantity is not None or
            self.additive is not None or
            super(Specimen_Container, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Specimen.Container', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen.Container')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Specimen.Container', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Specimen.Container'):
        super(Specimen_Container, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Specimen.Container')
    def exportChildren(self, outfile, level, namespace_='', name_='Specimen.Container', fromsubclass_=False, pretty_print=True):
        super(Specimen_Container, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.capacity is not None:
            self.capacity.export(outfile, level, namespace_, name_='capacity', pretty_print=pretty_print)
        if self.specimenQuantity is not None:
            self.specimenQuantity.export(outfile, level, namespace_, name_='specimenQuantity', pretty_print=pretty_print)
        if self.additive is not None:
            self.additive.export(outfile, level, namespace_, name_='additive', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Specimen.Container', mapping_=None):
        element = super(Specimen_Container, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.capacity is not None:
            capacity_ = self.capacity
            capacity_.to_etree(element, name_='capacity', mapping_=mapping_)
        if self.specimenQuantity is not None:
            specimenQuantity_ = self.specimenQuantity
            specimenQuantity_.to_etree(element, name_='specimenQuantity', mapping_=mapping_)
        if self.additive is not None:
            additive_ = self.additive
            additive_.to_etree(element, name_='additive', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Specimen_Container, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'capacity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.capacity = obj_
            obj_.original_tagname_ = 'capacity'
        elif nodeName_ == 'specimenQuantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.specimenQuantity = obj_
            obj_.original_tagname_ = 'specimenQuantity'
        elif nodeName_ == 'additive':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.additive = obj_
            obj_.original_tagname_ = 'additive'
        super(Specimen_Container, self).buildChildren(child_, node, nodeName_, True)
# end class Specimen_Container



from flask_restful import Resource
from server.health_fhir import health_Conformance
from server.common import tryton

institution = tryton.pool.get('party.party')

@tryton.transaction()
def find_institution():
    try:
        i = institution.search_read([('is_institution', '=', True)],
                                        limit=1,
                                        fields_names=['name'])[0]
    except:
        i = {'name': 'GNU Health'}
    return i.get('name', 'GNU Health')

# NOTE: Don't require authentication on this resource
class Conformance(Resource):
    def get(self):
        """Conformance interaction"""
        c = health_Conformance(publisher=find_institution())
        return c, 200

    def options(self):
        """Conformance interaction"""
        c = health_Conformance(publisher=find_institution())
        return c, 200

__all__=['Conformance']



from .base_classes import *
from .support_functions import *


class Observation(Resource):
    """Measurements and simple assertions made about a patient, device or
    other subject.If the element is present, it must have either a
    @value, an @id, or extensionsThe information determined as a
    result of making the observation, if the information has a
    simple value.The time or time-period the observed value is
    asserted as being true. For biological subjects - e.g. human
    patients - this is usually called the "physiologically relevant
    time". This is usually either the time of the procedure or of
    specimen collection, but very often the source of the date/time
    is not known, only the date/time itself."""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, name=None, valueQuantity=None, valueCodeableConcept=None, valueAttachment=None, valueRatio=None, valuePeriod=None, valueSampledData=None, valueString=None, interpretation=None, comments=None, appliesDateTime=None, appliesPeriod=None, issued=None, status=None, reliability=None, bodySite=None, method=None, identifier=None, subject=None, specimen=None, performer=None, referenceRange=None, related=None):
        self.original_tagname_ = None
        super(Observation, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.name = name
        self.valueQuantity = valueQuantity
        self.valueCodeableConcept = valueCodeableConcept
        self.valueAttachment = valueAttachment
        self.valueRatio = valueRatio
        self.valuePeriod = valuePeriod
        self.valueSampledData = valueSampledData
        self.valueString = valueString
        self.interpretation = interpretation
        self.comments = comments
        self.appliesDateTime = appliesDateTime
        self.appliesPeriod = appliesPeriod
        self.issued = issued
        self.status = status
        self.reliability = reliability
        self.bodySite = bodySite
        self.method = method
        self.identifier = identifier
        self.subject = subject
        self.specimen = specimen
        if performer is None:
            self.performer = []
        else:
            self.performer = performer
        if referenceRange is None:
            self.referenceRange = []
        else:
            self.referenceRange = referenceRange
        if related is None:
            self.related = []
        else:
            self.related = related
    def factory(*args_, **kwargs_):
        if Observation.subclass:
            return Observation.subclass(*args_, **kwargs_)
        else:
            return Observation(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_valueQuantity(self): return self.valueQuantity
    def set_valueQuantity(self, valueQuantity): self.valueQuantity = valueQuantity
    def get_valueCodeableConcept(self): return self.valueCodeableConcept
    def set_valueCodeableConcept(self, valueCodeableConcept): self.valueCodeableConcept = valueCodeableConcept
    def get_valueAttachment(self): return self.valueAttachment
    def set_valueAttachment(self, valueAttachment): self.valueAttachment = valueAttachment
    def get_valueRatio(self): return self.valueRatio
    def set_valueRatio(self, valueRatio): self.valueRatio = valueRatio
    def get_valuePeriod(self): return self.valuePeriod
    def set_valuePeriod(self, valuePeriod): self.valuePeriod = valuePeriod
    def get_valueSampledData(self): return self.valueSampledData
    def set_valueSampledData(self, valueSampledData): self.valueSampledData = valueSampledData
    def get_valueString(self): return self.valueString
    def set_valueString(self, valueString): self.valueString = valueString
    def get_interpretation(self): return self.interpretation
    def set_interpretation(self, interpretation): self.interpretation = interpretation
    def get_comments(self): return self.comments
    def set_comments(self, comments): self.comments = comments
    def get_appliesDateTime(self): return self.appliesDateTime
    def set_appliesDateTime(self, appliesDateTime): self.appliesDateTime = appliesDateTime
    def get_appliesPeriod(self): return self.appliesPeriod
    def set_appliesPeriod(self, appliesPeriod): self.appliesPeriod = appliesPeriod
    def get_issued(self): return self.issued
    def set_issued(self, issued): self.issued = issued
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_reliability(self): return self.reliability
    def set_reliability(self, reliability): self.reliability = reliability
    def get_bodySite(self): return self.bodySite
    def set_bodySite(self, bodySite): self.bodySite = bodySite
    def get_method(self): return self.method
    def set_method(self, method): self.method = method
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_specimen(self): return self.specimen
    def set_specimen(self, specimen): self.specimen = specimen
    def get_performer(self): return self.performer
    def set_performer(self, performer): self.performer = performer
    def add_performer(self, value): self.performer.append(value)
    def insert_performer(self, index, value): self.performer[index] = value
    def get_referenceRange(self): return self.referenceRange
    def set_referenceRange(self, referenceRange): self.referenceRange = referenceRange
    def add_referenceRange(self, value): self.referenceRange.append(value)
    def insert_referenceRange(self, index, value): self.referenceRange[index] = value
    def get_related(self): return self.related
    def set_related(self, related): self.related = related
    def add_related(self, value): self.related.append(value)
    def insert_related(self, index, value): self.related[index] = value
    def hasContent_(self):
        if (
            self.name is not None or
            self.valueQuantity is not None or
            self.valueCodeableConcept is not None or
            self.valueAttachment is not None or
            self.valueRatio is not None or
            self.valuePeriod is not None or
            self.valueSampledData is not None or
            self.valueString is not None or
            self.interpretation is not None or
            self.comments is not None or
            self.appliesDateTime is not None or
            self.appliesPeriod is not None or
            self.issued is not None or
            self.status is not None or
            self.reliability is not None or
            self.bodySite is not None or
            self.method is not None or
            self.identifier is not None or
            self.subject is not None or
            self.specimen is not None or
            self.performer or
            self.referenceRange or
            self.related or
            super(Observation, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Observation', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Observation')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Observation', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Observation'):
        super(Observation, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Observation')
    def exportChildren(self, outfile, level, namespace_='', name_='Observation', fromsubclass_=False, pretty_print=True):
        super(Observation, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.valueQuantity is not None:
            self.valueQuantity.export(outfile, level, namespace_, name_='valueQuantity', pretty_print=pretty_print)
        if self.valueCodeableConcept is not None:
            self.valueCodeableConcept.export(outfile, level, namespace_, name_='valueCodeableConcept', pretty_print=pretty_print)
        if self.valueAttachment is not None:
            self.valueAttachment.export(outfile, level, namespace_, name_='valueAttachment', pretty_print=pretty_print)
        if self.valueRatio is not None:
            self.valueRatio.export(outfile, level, namespace_, name_='valueRatio', pretty_print=pretty_print)
        if self.valuePeriod is not None:
            self.valuePeriod.export(outfile, level, namespace_, name_='valuePeriod', pretty_print=pretty_print)
        if self.valueSampledData is not None:
            self.valueSampledData.export(outfile, level, namespace_, name_='valueSampledData', pretty_print=pretty_print)
        if self.valueString is not None:
            self.valueString.export(outfile, level, namespace_, name_='valueString', pretty_print=pretty_print)
        if self.interpretation is not None:
            self.interpretation.export(outfile, level, namespace_, name_='interpretation', pretty_print=pretty_print)
        if self.comments is not None:
            self.comments.export(outfile, level, namespace_, name_='comments', pretty_print=pretty_print)
        if self.appliesDateTime is not None:
            self.appliesDateTime.export(outfile, level, namespace_, name_='appliesDateTime', pretty_print=pretty_print)
        if self.appliesPeriod is not None:
            self.appliesPeriod.export(outfile, level, namespace_, name_='appliesPeriod', pretty_print=pretty_print)
        if self.issued is not None:
            self.issued.export(outfile, level, namespace_, name_='issued', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.reliability is not None:
            self.reliability.export(outfile, level, namespace_, name_='reliability', pretty_print=pretty_print)
        if self.bodySite is not None:
            self.bodySite.export(outfile, level, namespace_, name_='bodySite', pretty_print=pretty_print)
        if self.method is not None:
            self.method.export(outfile, level, namespace_, name_='method', pretty_print=pretty_print)
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.specimen is not None:
            self.specimen.export(outfile, level, namespace_, name_='specimen', pretty_print=pretty_print)
        for performer_ in self.performer:
            performer_.export(outfile, level, namespace_, name_='performer', pretty_print=pretty_print)
        for referenceRange_ in self.referenceRange:
            referenceRange_.export(outfile, level, namespace_, name_='referenceRange', pretty_print=pretty_print)
        for related_ in self.related:
            related_.export(outfile, level, namespace_, name_='related', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Observation', mapping_=None):
        element = super(Observation, self).to_etree(parent_element, name_, mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.valueQuantity is not None:
            valueQuantity_ = self.valueQuantity
            valueQuantity_.to_etree(element, name_='valueQuantity', mapping_=mapping_)
        if self.valueCodeableConcept is not None:
            valueCodeableConcept_ = self.valueCodeableConcept
            valueCodeableConcept_.to_etree(element, name_='valueCodeableConcept', mapping_=mapping_)
        if self.valueAttachment is not None:
            valueAttachment_ = self.valueAttachment
            valueAttachment_.to_etree(element, name_='valueAttachment', mapping_=mapping_)
        if self.valueRatio is not None:
            valueRatio_ = self.valueRatio
            valueRatio_.to_etree(element, name_='valueRatio', mapping_=mapping_)
        if self.valuePeriod is not None:
            valuePeriod_ = self.valuePeriod
            valuePeriod_.to_etree(element, name_='valuePeriod', mapping_=mapping_)
        if self.valueSampledData is not None:
            valueSampledData_ = self.valueSampledData
            valueSampledData_.to_etree(element, name_='valueSampledData', mapping_=mapping_)
        if self.valueString is not None:
            valueString_ = self.valueString
            valueString_.to_etree(element, name_='valueString', mapping_=mapping_)
        if self.interpretation is not None:
            interpretation_ = self.interpretation
            interpretation_.to_etree(element, name_='interpretation', mapping_=mapping_)
        if self.comments is not None:
            comments_ = self.comments
            comments_.to_etree(element, name_='comments', mapping_=mapping_)
        if self.appliesDateTime is not None:
            appliesDateTime_ = self.appliesDateTime
            appliesDateTime_.to_etree(element, name_='appliesDateTime', mapping_=mapping_)
        if self.appliesPeriod is not None:
            appliesPeriod_ = self.appliesPeriod
            appliesPeriod_.to_etree(element, name_='appliesPeriod', mapping_=mapping_)
        if self.issued is not None:
            issued_ = self.issued
            issued_.to_etree(element, name_='issued', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.reliability is not None:
            reliability_ = self.reliability
            reliability_.to_etree(element, name_='reliability', mapping_=mapping_)
        if self.bodySite is not None:
            bodySite_ = self.bodySite
            bodySite_.to_etree(element, name_='bodySite', mapping_=mapping_)
        if self.method is not None:
            method_ = self.method
            method_.to_etree(element, name_='method', mapping_=mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.specimen is not None:
            specimen_ = self.specimen
            specimen_.to_etree(element, name_='specimen', mapping_=mapping_)
        for performer_ in self.performer:
            performer_.to_etree(element, name_='performer', mapping_=mapping_)
        for referenceRange_ in self.referenceRange:
            referenceRange_.to_etree(element, name_='referenceRange', mapping_=mapping_)
        for related_ in self.related:
            related_.to_etree(element, name_='related', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Observation, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'name':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'valueQuantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.valueQuantity = obj_
            obj_.original_tagname_ = 'valueQuantity'
        elif nodeName_ == 'valueCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.valueCodeableConcept = obj_
            obj_.original_tagname_ = 'valueCodeableConcept'
        elif nodeName_ == 'valueAttachment':
            obj_ = Attachment.factory()
            obj_.build(child_)
            self.valueAttachment = obj_
            obj_.original_tagname_ = 'valueAttachment'
        elif nodeName_ == 'valueRatio':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.valueRatio = obj_
            obj_.original_tagname_ = 'valueRatio'
        elif nodeName_ == 'valuePeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.valuePeriod = obj_
            obj_.original_tagname_ = 'valuePeriod'
        elif nodeName_ == 'valueSampledData':
            obj_ = SampledData.factory()
            obj_.build(child_)
            self.valueSampledData = obj_
            obj_.original_tagname_ = 'valueSampledData'
        elif nodeName_ == 'valueString':
            obj_ = string.factory()
            obj_.build(child_)
            self.valueString = obj_
            obj_.original_tagname_ = 'valueString'
        elif nodeName_ == 'interpretation':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.interpretation = obj_
            obj_.original_tagname_ = 'interpretation'
        elif nodeName_ == 'comments':
            obj_ = string.factory()
            obj_.build(child_)
            self.comments = obj_
            obj_.original_tagname_ = 'comments'
        elif nodeName_ == 'appliesDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.appliesDateTime = obj_
            obj_.original_tagname_ = 'appliesDateTime'
        elif nodeName_ == 'appliesPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.appliesPeriod = obj_
            obj_.original_tagname_ = 'appliesPeriod'
        elif nodeName_ == 'issued':
            obj_ = instant.factory()
            obj_.build(child_)
            self.issued = obj_
            obj_.original_tagname_ = 'issued'
        elif nodeName_ == 'status':
            obj_ = ObservationStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'reliability':
            obj_ = ObservationReliability.factory()
            obj_.build(child_)
            self.reliability = obj_
            obj_.original_tagname_ = 'reliability'
        elif nodeName_ == 'bodySite':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.bodySite = obj_
            obj_.original_tagname_ = 'bodySite'
        elif nodeName_ == 'method':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.method = obj_
            obj_.original_tagname_ = 'method'
        elif nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'specimen':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.specimen = obj_
            obj_.original_tagname_ = 'specimen'
        elif nodeName_ == 'performer':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.performer.append(obj_)
            obj_.original_tagname_ = 'performer'
        elif nodeName_ == 'referenceRange':
            obj_ = Observation_ReferenceRange.factory()
            obj_.build(child_)
            self.referenceRange.append(obj_)
            obj_.original_tagname_ = 'referenceRange'
        elif nodeName_ == 'related':
            obj_ = Observation_Related.factory()
            obj_.build(child_)
            self.related.append(obj_)
            obj_.original_tagname_ = 'related'
        super(Observation, self).buildChildren(child_, node, nodeName_, True)
# end class Observation


class Observation_ReferenceRange(BackboneElement):
    """Measurements and simple assertions made about a patient, device or
    other subject."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, low=None, high=None, meaning=None, age=None):
        self.original_tagname_ = None
        super(Observation_ReferenceRange, self).__init__(id, extension, modifierExtension, )
        self.low = low
        self.high = high
        self.meaning = meaning
        self.age = age
    def factory(*args_, **kwargs_):
        if Observation_ReferenceRange.subclass:
            return Observation_ReferenceRange.subclass(*args_, **kwargs_)
        else:
            return Observation_ReferenceRange(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_low(self): return self.low
    def set_low(self, low): self.low = low
    def get_high(self): return self.high
    def set_high(self, high): self.high = high
    def get_meaning(self): return self.meaning
    def set_meaning(self, meaning): self.meaning = meaning
    def get_age(self): return self.age
    def set_age(self, age): self.age = age
    def hasContent_(self):
        if (
            self.low is not None or
            self.high is not None or
            self.meaning is not None or
            self.age is not None or
            super(Observation_ReferenceRange, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Observation.ReferenceRange', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Observation.ReferenceRange')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Observation.ReferenceRange', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Observation.ReferenceRange'):
        super(Observation_ReferenceRange, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Observation.ReferenceRange')
    def exportChildren(self, outfile, level, namespace_='', name_='Observation.ReferenceRange', fromsubclass_=False, pretty_print=True):
        super(Observation_ReferenceRange, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.low is not None:
            self.low.export(outfile, level, namespace_, name_='low', pretty_print=pretty_print)
        if self.high is not None:
            self.high.export(outfile, level, namespace_, name_='high', pretty_print=pretty_print)
        if self.meaning is not None:
            self.meaning.export(outfile, level, namespace_, name_='meaning', pretty_print=pretty_print)
        if self.age is not None:
            self.age.export(outfile, level, namespace_, name_='age', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Observation.ReferenceRange', mapping_=None):
        element = super(Observation_ReferenceRange, self).to_etree(parent_element, name_, mapping_)
        if self.low is not None:
            low_ = self.low
            low_.to_etree(element, name_='low', mapping_=mapping_)
        if self.high is not None:
            high_ = self.high
            high_.to_etree(element, name_='high', mapping_=mapping_)
        if self.meaning is not None:
            meaning_ = self.meaning
            meaning_.to_etree(element, name_='meaning', mapping_=mapping_)
        if self.age is not None:
            age_ = self.age
            age_.to_etree(element, name_='age', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Observation_ReferenceRange, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'low':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.low = obj_
            obj_.original_tagname_ = 'low'
        elif nodeName_ == 'high':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.high = obj_
            obj_.original_tagname_ = 'high'
        elif nodeName_ == 'meaning':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.meaning = obj_
            obj_.original_tagname_ = 'meaning'
        elif nodeName_ == 'age':
            obj_ = Range.factory()
            obj_.build(child_)
            self.age = obj_
            obj_.original_tagname_ = 'age'
        super(Observation_ReferenceRange, self).buildChildren(child_, node, nodeName_, True)
# end class Observation_ReferenceRange


class Observation_Related(BackboneElement):
    """Measurements and simple assertions made about a patient, device or
    other subject."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, type_=None, target=None):
        self.original_tagname_ = None
        super(Observation_Related, self).__init__(id, extension, modifierExtension, )
        self.type_ = type_
        self.target = target
    def factory(*args_, **kwargs_):
        if Observation_Related.subclass:
            return Observation_Related.subclass(*args_, **kwargs_)
        else:
            return Observation_Related(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_target(self): return self.target
    def set_target(self, target): self.target = target
    def hasContent_(self):
        if (
            self.type_ is not None or
            self.target is not None or
            super(Observation_Related, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Observation.Related', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Observation.Related')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Observation.Related', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Observation.Related'):
        super(Observation_Related, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Observation.Related')
    def exportChildren(self, outfile, level, namespace_='', name_='Observation.Related', fromsubclass_=False, pretty_print=True):
        super(Observation_Related, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.target is not None:
            self.target.export(outfile, level, namespace_, name_='target', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Observation.Related', mapping_=None):
        element = super(Observation_Related, self).to_etree(parent_element, name_, mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.target is not None:
            target_ = self.target
            target_.to_etree(element, name_='target', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Observation_Related, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = ObservationRelationshipType.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'target':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.target = obj_
            obj_.original_tagname_ = 'target'
        super(Observation_Related, self).buildChildren(child_, node, nodeName_, True)
# end class Observation_Related


class ObservationReliability(Element):
    """Codes that provide an estimate of the degree to which quality issues
    have impacted on the value of an observationIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ObservationReliability, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ObservationReliability.subclass:
            return ObservationReliability.subclass(*args_, **kwargs_)
        else:
            return ObservationReliability(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ObservationReliability, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ObservationReliability', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ObservationReliability')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ObservationReliability', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ObservationReliability'):
        super(ObservationReliability, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ObservationReliability')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ObservationReliability', fromsubclass_=False, pretty_print=True):
        super(ObservationReliability, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ObservationReliability', mapping_=None):
        element = super(ObservationReliability, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ObservationReliability, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ObservationReliability, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ObservationReliability


class ObservationStatus(Element):
    """Codes providing the status of an observationIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ObservationStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ObservationStatus.subclass:
            return ObservationStatus.subclass(*args_, **kwargs_)
        else:
            return ObservationStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ObservationStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ObservationStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ObservationStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ObservationStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ObservationStatus'):
        super(ObservationStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ObservationStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ObservationStatus', fromsubclass_=False, pretty_print=True):
        super(ObservationStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ObservationStatus', mapping_=None):
        element = super(ObservationStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ObservationStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ObservationStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ObservationStatus


class ObservationRelationshipType(Element):
    """Codes specifying how two observations are relatedIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ObservationRelationshipType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ObservationRelationshipType.subclass:
            return ObservationRelationshipType.subclass(*args_, **kwargs_)
        else:
            return ObservationRelationshipType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ObservationRelationshipType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ObservationRelationshipType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ObservationRelationshipType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ObservationRelationshipType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ObservationRelationshipType'):
        super(ObservationRelationshipType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ObservationRelationshipType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ObservationRelationshipType', fromsubclass_=False, pretty_print=True):
        super(ObservationRelationshipType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ObservationRelationshipType', mapping_=None):
        element = super(ObservationRelationshipType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ObservationRelationshipType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ObservationRelationshipType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ObservationRelationshipType



from .base_classes import *
from .support_functions import *


class Alert(Resource):
    """Prospective warnings of potential issues when providing care to the
    patient.If the element is present, it must have either a @value,
    an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, category=None, status=None, subject=None, author=None, note=None):
        self.original_tagname_ = None
        super(Alert, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.category = category
        self.status = status
        self.subject = subject
        self.author = author
        self.note = note
    def factory(*args_, **kwargs_):
        if Alert.subclass:
            return Alert.subclass(*args_, **kwargs_)
        else:
            return Alert(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_category(self): return self.category
    def set_category(self, category): self.category = category
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_author(self): return self.author
    def set_author(self, author): self.author = author
    def get_note(self): return self.note
    def set_note(self, note): self.note = note
    def hasContent_(self):
        if (
            self.identifier or
            self.category is not None or
            self.status is not None or
            self.subject is not None or
            self.author is not None or
            self.note is not None or
            super(Alert, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Alert', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Alert')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Alert', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Alert'):
        super(Alert, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Alert')
    def exportChildren(self, outfile, level, namespace_='', name_='Alert', fromsubclass_=False, pretty_print=True):
        super(Alert, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.category is not None:
            self.category.export(outfile, level, namespace_, name_='category', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.author is not None:
            self.author.export(outfile, level, namespace_, name_='author', pretty_print=pretty_print)
        if self.note is not None:
            self.note.export(outfile, level, namespace_, name_='note', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Alert', mapping_=None):
        element = super(Alert, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.category is not None:
            category_ = self.category
            category_.to_etree(element, name_='category', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.author is not None:
            author_ = self.author
            author_.to_etree(element, name_='author', mapping_=mapping_)
        if self.note is not None:
            note_ = self.note
            note_.to_etree(element, name_='note', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Alert, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'category':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.category = obj_
            obj_.original_tagname_ = 'category'
        elif nodeName_ == 'status':
            obj_ = AlertStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'author':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.author = obj_
            obj_.original_tagname_ = 'author'
        elif nodeName_ == 'note':
            obj_ = string.factory()
            obj_.build(child_)
            self.note = obj_
            obj_.original_tagname_ = 'note'
        super(Alert, self).buildChildren(child_, node, nodeName_, True)
# end class Alert


class AlertStatus(Element):
    """Indicates whether this alert is active and needs to be displayed to
    a user, or whether it is no longer needed or entered in errorIf
    the element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(AlertStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if AlertStatus.subclass:
            return AlertStatus.subclass(*args_, **kwargs_)
        else:
            return AlertStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(AlertStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='AlertStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='AlertStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='AlertStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='AlertStatus'):
        super(AlertStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='AlertStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='AlertStatus', fromsubclass_=False, pretty_print=True):
        super(AlertStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='AlertStatus', mapping_=None):
        element = super(AlertStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(AlertStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(AlertStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class AlertStatus
